# -*- coding: utf-8 -*-
#
#       settings_assistant.py
#
#       Copyright © 2011, 2014 Brandon Invergo <brandon@invergo.net>
#       Copyright © 2009, 2010 Per Liedman <per@liedman.net>
#
#       This file is part of Grotesque.
#
#       Grotesque is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Grotesque is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Grotesque.  If not, see <http://www.gnu.org/licenses/>.


from gi.repository import Gtk

from grotesque.which import whichall


class SettingsAssistant:
    '''This class creates the settings wizard which launches the first time
    Grotesque is run.

    '''
    SUGGESTED_INTERPRETERS = [
        ('zcode', ['gargoyle', 'gargoyle-free', 'gargoyle-frotz', 'frotz',
                   'zoom']),
        ('tads2', ['gargoyle', 'gargoyle-free', 'gargoyle-tadsr', 'tadsr',
                   'qtads']),
        ('tads3', ['gargoyle', 'gargoyle-free', 'gargoyle-tadsr', 'tadsr',
                   'qtads']),
        ('glulx', ['gargoyle', 'gargoyle-free', 'gargoyle-git', 'git']),
        ('adrift', ['gargoyle', 'gargoyle-free', 'gargoyle-scare', 'scare']),
        ('hugo', ['gargoyle', 'gargoyle-free', 'hugor']),
        ('alan2', ['gargoyle', 'gargoyle-free', 'xarun', 'arun', 'winarun']),
        ('alan3', ['gargoyle', 'gargoyle-free'])]

    INTRO_TEXT = ('It appears this is the first time you have run '
                  '<b><i>Grotesque</i></b>.\n\nPlease take a moment and let '
                  'this assistant guide you through setting up the '
                  'application.')

    INTERPRETERS_INFO = ('To play games using <b><i>Grotesque</i></b>, you '
                         'need to configure an interpreter for each game '
                         'format you wish to play.\n\nAn interpreter is a '
                         'program that knows how to play a certain game '
                         'format.\n\nBelow, a number of formats are listed, '
                         'and Grotesque has attempted to guess which '
                         'interpreter to use, based on the programs installed '
                         'on your computer.')

    COMPLETED_TEXT = 'Setup is complete. Enjoy <b><i>Grotesque</i></b>!'

    def __init__(self, settings, parent=None, on_finish=None):
        self.settings = settings
        self.assistant = Gtk.Assistant()
        self.on_finish = on_finish
        self.selected_interpreters = {}
        if parent is not None:
            self.assistant.set_transient_for(parent)

        intro_page = self.create_welcome_page()
        self.assistant.append_page(intro_page)
        self.assistant.set_page_type(intro_page, Gtk.AssistantPageType.INTRO)
        self.assistant.set_page_title(intro_page, 'Welcome to Grotesque')
        self.assistant.set_page_complete(intro_page, True)

        terp_page = self.create_interpreter_page()
        self.assistant.append_page(terp_page)
        self.assistant.set_page_type(terp_page, Gtk.AssistantPageType.CONTENT)
        self.assistant.set_page_title(terp_page, 'Select interpreters')
        self.assistant.set_page_complete(terp_page, True)

        summary_page = self.create_summary_page()
        self.assistant.append_page(summary_page)
        self.assistant.set_page_type(summary_page,
                                     Gtk.AssistantPageType.SUMMARY)
        self.assistant.set_page_title(summary_page, 'Setup complete')
        self.assistant.set_page_complete(summary_page, True)

        self.assistant.connect('close', self.on_close)
        self.assistant.connect('cancel', self.on_cancel)
        self.assistant.show_all()

    def on_cancel(self, assistant):
        if self.on_finish is not None:
            self.on_finish()
        self.assistant.destroy()

    def on_close(self, assistant):
        for format in self.selected_interpreters:
            self.settings.set_launcher(format,
                                       self.selected_interpreters[format])
        if self.on_finish is not None:
            self.on_finish()
        self.assistant.destroy()

    def create_welcome_page(self):
        w = Gtk.Label()
        w.set_line_wrap(True)
        w.set_justify(Gtk.Justification.LEFT)
        w.set_markup(self.INTRO_TEXT)
        w.set_padding(16, 24)
        return w

    def create_interpreter_page(self):
        vbox = Gtk.VBox()
        info_label = Gtk.Label()
        info_label.set_line_wrap(True)
        info_label.set_justify(Gtk.Justification.LEFT)
        info_label.set_markup(self.INTERPRETERS_INFO)
        vbox.pack_start(info_label, False, False, 8)
        for (format, interpreters) in self.SUGGESTED_INTERPRETERS:
            hbox = Gtk.HBox()
            label = Gtk.Label(label=format)
            label.set_justify(Gtk.Justification.LEFT)
            hbox.pack_start(label, True, False, 0)
            combo = Gtk.ComboBoxText.new_with_entry()
            first = True
            for terp in interpreters:
                for exe in whichall(terp):
                    combo.append_text(exe)
                    if first:
                        self.selected_interpreters[format] = exe
                        combo.set_active(0)
                        first = False
            combo.connect('changed', self.on_interpreter_change, format)
            hbox.pack_start(combo, False, False, 16)
            vbox.pack_start(hbox, False, False, 0)
        return vbox

    def on_interpreter_change(self, widget, format):
        self.selected_interpreters[format] = widget.get_active_text()

    def create_summary_page(self):
        w = Gtk.Label()
        w.set_line_wrap(True)
        w.set_justify(Gtk.Justification.LEFT)
        w.set_markup(self.COMPLETED_TEXT)
        w.set_padding(16, 24)
        return w
