# -*- coding: utf-8 -*-
#
#       infoview.py
#
#       Copyright © 2011, 2014 Brandon Invergo <brandon@invergo.net>
#
#       This file is part of Grotesque.
#
#       Grotesque is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Grotesque is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Grotesque.  If not, see <http://www.gnu.org/licenses/>.


import os.path

from gi.repository import Gtk, Pango


class InfoView(Gtk.TextView):
    '''This class implements a widget which displays story information.

    '''
    def __init__(self):
        # Create all of the tags which will markup the text:
        # Title: XX-large & bold
        title_tag = Gtk.TextTag(name='title')
        title_tag.set_property('scale', 1.2 ** 3)
        title_tag.set_property('weight', Pango.Weight.BOLD)
        # Headline: X-Large & italic
        headline_tag = Gtk.TextTag(name='headline')
        headline_tag.set_property('scale', 1.2 ** 2)
        headline_tag.set_property('style', Pango.Style.ITALIC)
        # Series: X-Large
        series_tag = Gtk.TextTag(name='series')
        series_tag.set_property('scale', 1.2 ** 2)
        # Author: Large
        author_tag = Gtk.TextTag(name='author')
        author_tag.set_property('scale', 1.2)
        # Info: Small & italic
        info_tag = Gtk.TextTag(name='info')
        info_tag.set_property('scale', 0.83)
        info_tag.set_property('style', Pango.Style.ITALIC)
        # Primary IFID: bold Info
        ifid_tag = Gtk.TextTag(name='ifid')
        ifid_tag.set_property('scale', 0.83)
        ifid_tag.set_property('style', Pango.Style.ITALIC)
        ifid_tag.set_property('weight', Pango.Weight.BOLD)
        # Plain text bold
        bold_tag = Gtk.TextTag(name='bold')
        bold_tag.set_property('weight', Pango.Weight.BOLD)
        # Plain text italics
        italics_tag = Gtk.TextTag(name='italics')
        italics_tag.set_property('style', Pango.Style.ITALIC)
        text_buffer = Gtk.TextBuffer()
        tag_table = text_buffer.get_tag_table()
        for tag in [title_tag, headline_tag, series_tag, author_tag, info_tag,
                    ifid_tag, bold_tag, italics_tag]:
            tag_table.add(tag)
        super(InfoView, self).__init__(buffer=text_buffer)

    def render_story(self, story):
        '''This method inserts the story information in the text buffer and
        applies the markup tags to properly format it.

        '''
        self.clear()
        text_buffer = self.get_buffer()
        text_iter = text_buffer.get_start_iter()
        tag_table = text_buffer.get_tag_table()
        biblio = story.bibliographic
        text_buffer.insert_with_tags(
            text_iter, u'{0}\n'.format(biblio["title"]),
            tag_table.lookup('title'))
        headline = biblio.get('headline')
        if headline not in ['None', '']:
            text_buffer.insert_with_tags(
                text_iter, u'{0}\n'.format(headline),
                tag_table.lookup('headline'))
        series = biblio.get('series')
        seriesnumber = biblio.get('seriesnumber')
        if series not in ['None', '']:
            text_buffer.insert_with_tags(
                text_iter, u'{0} (part {1})\n'.format(
                    series, seriesnumber), tag_table.lookup('series'))
        text_buffer.insert_with_tags(
            text_iter, u'by {0}\n'.format(biblio['author']),
            tag_table.lookup('author'))
        group = biblio.get('group')
        if group not in ['None', '']:
            text_buffer.insert(
                text_iter, u'({0})\n'.format(group))
        annot = story.annotation.get('ifdb')
        starrating = annot.get('starrating')
        if starrating is not None:
            rating_pixbuf = self.get_rating_pixbuf(starrating)
            text_buffer.insert_pixbuf(text_iter, rating_pixbuf)
        text_buffer.insert(text_iter, '\n')
        description = biblio.get('description')
        text_buffer.insert(text_iter, u'\n')
        if description is not None and description != "":
            cut_description = self.format_html(description)
            for chunk in cut_description:
                if chunk[1] is None:
                    text_buffer.insert(text_iter, chunk[0])
                else:
                    text_buffer.insert_with_tags(
                        text_iter, chunk[0], tag_table.lookup(chunk[1]))
        text_buffer.insert(text_iter, u'\n')
        firstpublished = biblio.get('firstpublished')
        text_buffer.insert_with_tags(
            text_iter, u'\nFirst published: {0}\n'.format(
                firstpublished), tag_table.lookup('info'))
        genre = biblio.get('genre')
        text_buffer.insert_with_tags(
            text_iter, u'Genre: {0}\n'.format(genre),
            tag_table.lookup('info'))
        forgiveness = biblio.get('forgiveness')
        text_buffer.insert_with_tags(
            text_iter, u'Forgiveness: {0}\n'.format(
                forgiveness), tag_table.lookup('info'))
        text_buffer.insert_with_tags(
            text_iter, u'Format: {0}\n'.format(story.format),
            tag_table.lookup('info'))
        ifid_header = u'IFID(s):\n'
        text_buffer.insert_with_tags(
            text_iter, ifid_header, tag_table.lookup('info'))
        primary_ifid = u'\t{0}\n'.format(story.ifid_list[0])
        text_buffer.insert_with_tags(
            text_iter, primary_ifid, tag_table.lookup('ifid'))
        if len(story.ifid_list) > 1:
            ifid_list_str = '\n\t'.join(story.ifid_list[1:])
            ifid_str = ''.join(['\t', ifid_list_str, '\n'])
            text_buffer.insert_with_tags(
                text_iter, ifid_str, tag_table.lookup('info'))
        ifdb = story.annotation.get("ifdb")
        if ifdb is not None:
            tuid = ifdb.get('tuid')
            text_buffer.insert_with_tags(
                text_iter, u'TUID: {0}\n'.format(tuid),
                tag_table.lookup('info'))

    def clear(self):
        '''This is a helper method to clear the contents of the view.

        '''
        text_buffer = self.get_buffer()
        text_buffer.set_text('')

    def format_html(self, text):
        tag_formats = {'i': 'italics', 'b': 'bold'}
        text = text.replace('<br/>', '\n')
        text = text.replace('<br />', '\n')
        text = text.replace('<p>', '\n\n')
        text = text.replace('</p>', '')
        cut_text = []
        index = 0
        while index < len(text):
            start = text[index:].find('<')
            if start >= 0:
                cut_text.append((text[index:start], None))
                tag = text[start+1]
                if tag not in tag_formats:
                    continue
                end = text[start+3:].find('</')
                if end >= 0:
                    cut_text.append((text[start+3:end], tag_formats[tag]))
                else:
                    cut_text.append((text[start+3:end], None))
                index = index + start + 3 + end + 4
            else:
                cut_text.append((text[index:], None))
                break
        return cut_text

    def get_rating_pixbuf(self, rating):
        '''This method fetches the star rating image given a textual
        description of it ("2", "3.5", etc.).

        '''
        rating_str = rating.replace('.', '-')
        rating_file = os.path.join(os.path.dirname(__file__), '..',
                                   '..', '..', 'data',
                                   'stars_{0}.png'.format(rating_str))
        rating_image = Gtk.Image.new_from_file(rating_file)
        return rating_image.get_pixbuf()
