# -*- coding: utf-8 -*-
#
#       library.py
#
#       Copyright © 2011, 2014 Brandon Invergo <brandon@invergo.net>
#       Copyright © 2009, 2010 Per Liedman <per@liedman.net>
#
#       This file is part of Grotesque.
#
#       Grotesque is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Grotesque is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Grotesque.  If not, see <http://www.gnu.org/licenses/>.


import cPickle
import os.path
import time

from gi.repository import Gtk, GdkPixbuf, Pango, GObject

from filterstore import FilterStore
from ..threads.storyimportthread import CoverImportThread
from ..dialogs.progressdialog import ProgressDialog
from treatyofbabel import ifiction, ifstory
from treatyofbabel.utils._imgfuncs import get_png_dim, get_jpeg_dim


class Library:
    '''This class stores the user's library of interactive fiction.

    '''
    def __init__(self):
        # The main list store contains the following columns:
        # 0: played state, 1: title, 2: author, 3: language, 4: headline,
        # 5: publishing date, 6: genre, 7: group, 8: series, 9: series number,
        # 10: forgiveness, 11: star rating image, 12: star rating text,
        # 13: description, 14: story file location, 15: story object,
        # 16: the text weight to render the row at
        self.list_store = Gtk.ListStore(bool, str, str, str, str, str, str,
                                        str, str, str, str, GdkPixbuf.Pixbuf,
                                        str, str, str, object, Pango.Weight)
        # The following Filterstores keep track of which authors, years, etc
        # are currently represented in the library so the user may filter it by
        # them.
        self.author_store = FilterStore()
        self.author_store.append(['(All)'])
        self.year_store = FilterStore()
        self.year_store.append(['(All)'])
        self.genre_store = FilterStore()
        self.genre_store.append(['(All)'])
        self.group_store = FilterStore()
        self.group_store.append(['(All)'])
        self.series_store = FilterStore()
        self.series_store.append(['(All)'])
        self.forgiveness_store = FilterStore()
        self.forgiveness_store.append(['(All)'])
        self.rating_store = FilterStore(True)
        # This list keeps track of which files are already in the library.
        self._files = []

    def add_entry(self, entry):
        '''This method handles adding stories to the library.

        '''
        # Avoid duplicate entries.
        if entry.story_file in self._files:
            return
        # For some of the fields, it's better to have some default text rather
        # than no text.
        if not entry.bibliographic["author"]:
            entry.bibliographic["author"] = 'Unknown'
        if not entry.bibliographic["firstpublished"]:
            entry.bibliographic["firstpublished"] = 'Unknown'
        if not entry.bibliographic["genre"]:
            entry.bibliographic["genre"] = 'Unknown'
        if not entry.bibliographic["group"]:
            entry.bibliographic["group"] = 'None'
        if not entry.bibliographic["series"]:
            entry.bibliographic["series"] = 'None'
        if not entry.bibliographic["forgiveness"]:
            entry.bibliographic["forgiveness"] = 'Unknown'
        if (not entry.annotation.get("ifdb")
                or not entry.annotation["ifdb"].get("starrating")):
            entry.add_annotation("ifdb", {"starrating": '0'})
        # Set the font weight depending on whether the story has been played or
        # not
        if entry.annotation["grotesque"]["played"] == "False":
            text_weight = Pango.Weight.BOLD
        else:
            text_weight = Pango.Weight.NORMAL
        # Get the star rating image
        rating_image = self.get_rating_pixbuf(
            entry.annotation["ifdb"]["starrating"])
        # Add the story to the main list.
        self.list_store.append([entry.annotation["grotesque"]["played"],
                                entry.bibliographic["title"],
                                entry.bibliographic["author"],
                                entry.bibliographic["language"],
                                entry.bibliographic["headline"],
                                entry.bibliographic["firstpublished"],
                                entry.bibliographic["genre"],
                                entry.bibliographic["group"],
                                entry.bibliographic["series"],
                                entry.bibliographic["seriesnumber"],
                                entry.bibliographic["forgiveness"],
                                rating_image,
                                entry.annotation["ifdb"]["starrating"],
                                entry.bibliographic["description"],
                                entry.story_file,
                                entry, text_weight])
        # Add it to the list of known files.
        self._files.append(entry.story_file)
        # Parse the authors text to get a list of all the authors involved.
        authors = self.parse_list_str(entry.bibliographic["author"])
        for author in authors:
            # Only add the author's real name to the filter (no pen names).
            author_real_name = author.split('(')[0]
            if author_real_name == '':
                continue
            self.author_store.add_item(author_real_name.strip())
        # We only filter by year, so if month/day are included, cut them out.
        year = entry.bibliographic["firstpublished"].split('-')[0]
        self.year_store.add_item(year)
        # Parse genre to get a list of all genres named.
        genres = self.parse_list_str(entry.bibliographic["genre"])
        for genre in genres:
            self.genre_store.add_item(genre.strip())
        self.group_store.add_item(entry.bibliographic["group"])
        self.series_store.add_item(entry.bibliographic["series"])
        self.forgiveness_store.add_item(entry.bibliographic["forgiveness"])
        self.rating_store.add_item(entry.annotation["ifdb"]["starrating"],
                                   rating_image)

    def edit_entry(self, row_iter, biblio_info):
        '''This method handles the editing of a story entry.

        '''
        entry = self.get_story(row_iter)
        if entry.bibliographic["title"] != biblio_info['Title']:
            if not biblio_info['Title']:
                entry.title = 'Unknown'
            else:
                entry.title = biblio_info['Title']
            self.list_store.set_value(row_iter, 1, entry.title)
        if entry.bibliographic["author"] != biblio_info['Author']:
            if entry.bibliographic["author"]:
                authors = self.parse_list_str(entry.bibliographic["author"])
                for author in authors:
                    author_real_name = author.split('(')[0]
                    if author_real_name == '':
                        continue
                    self.author_store.remove_item(author_real_name.strip())
            if not biblio_info['Author']:
                entry.bibliographic["author"] = 'Unknown'
            else:
                entry.bibliographic["author"] = biblio_info['Author']
            self.list_store.set_value(row_iter, 2,
                                      entry.bibliographic["author"])
            authors = self.parse_list_str(entry.bibliographic["author"])
            for author in authors:
                author_real_name = author.split('(')[0]
                if author_real_name == '':
                    continue
                self.author_store.add_item(author_real_name.strip())
        if entry.bibliographic["language"] != biblio_info['Language']:
            if not biblio_info['Language']:
                entry.bibliographic["language"] = None
            else:
                entry.bibliographic["language"] = biblio_info['Language']
            self.list_store.set_value(row_iter, 3,
                                      entry.bibliographic["language"])
        if entry.bibliographic["headline"] != biblio_info['Headline']:
            if not biblio_info['Headline']:
                entry.bibliographic["headline"] = None
            else:
                entry.bibliographic["headline"] = biblio_info['Headline']
            self.list_store.set_value(row_iter, 4,
                                      entry.bibliographic["headline"])
        if entry.bibliographic["firstpublished"] != biblio_info['First published']:
            if entry.bibliographic["firstpublished"]:
                year = entry.bibliographic["firstpublished"].split('-')[0]
                self.year_store.remove_item(year)
            if not biblio_info['First published']:
                entry.bibliographic["firstpublished"] = 'Unknown'
            else:
                entry.bibliographic["firstpublished"] = biblio_info['First published']
            self.list_store.set_value(row_iter, 5,
                                      entry.bibliographic["firstpublished"])
            year = entry.bibliographic["firstpublished"].split('-')[0]
            self.year_store.add_item(year)
        if entry.bibliographic["genre"] != biblio_info['Genre']:
            if entry.bibliographic["genre"]:
                genres = self.parse_list_str(entry.bibliographic["genre"])
                for genre in genres:
                    self.genre_store.remove_item(genre.strip())
            if not biblio_info['Genre']:
                entry.bibliographic["genre"] = 'Unknown'
            else:
                entry.bibliographic["genre"] = biblio_info['Genre']
            self.list_store.set_value(row_iter, 6,
                                      entry.bibliographic["genre"])
            genres = self.parse_list_str(entry.bibliographic["genre"])
            for genre in genres:
                self.genre_store.add_item(genre.strip())
        if entry.bibliographic["group"] != biblio_info['Group']:
            if entry.bibliographic["group"]:
                self.group_store.remove_item(entry.bibliographic["group"])
            if not biblio_info['Group']:
                entry.bibliographic["group"] = 'None'
            else:
                entry.bibliographic["group"] = biblio_info['Group']
            self.list_store.set_value(row_iter, 7,
                                      entry.bibliographic["group"])
            self.group_store.add_item(entry.bibliographic["group"])
        if entry.bibliographic["series"] != biblio_info['Series']:
            if entry.bibliographic["series"]:
                self.series_store.remove_item(entry.bibliographic["series"])
            if not biblio_info['Series']:
                entry.bibliographic["series"] = 'None'
            else:
                entry.bibliographic["series"] = biblio_info['Series']
            self.list_store.set_value(row_iter, 8,
                                      entry.bibliographic["series"])
            self.series_store.add_item(entry.bibliographic["series"])
        if entry.bibliographic["seriesnumber"] != biblio_info['Series number']:
            if not biblio_info['Series number']:
                entry.bibliographic["seriesnumber"] = None
            else:
                entry.bibliographic["seriesnumber"] = biblio_info['Series number']
            self.list_store.set_value(row_iter, 9,
                                      entry.bibliographic["seriesnumber"])
        if entry.bibliographic["forgiveness"] != biblio_info['Forgiveness']:
            if entry.bibliographic["forgiveness"]:
                self.forgiveness_store.remove_item(
                    entry.bibliographic["forgiveness"])
            if not biblio_info['Forgiveness']:
                entry.bibliographic["forgiveness"] = 'Unknown'
            else:
                entry.bibliographic["forgiveness"] = biblio_info['Forgiveness']
            self.list_store.set_value(row_iter, 10,
                                      entry.bibliographic["forgiveness"])
            self.forgiveness_store.add_item(entry.bibliographic["forgiveness"])
        if entry.annotation["ifdb"]["starrating"] != biblio_info['Rating']:
            if entry.annotation["ifdb"]["starrating"]:
                self.rating_store.remove_item(
                    entry.annotation["ifdb"]["starrating"])
            if not biblio_info['Rating']:
                entry.annotation["ifdb"]["starrating"] = "0"
            else:
                entry.annotation["ifdb"]["starrating"] = biblio_info['Rating']
            self.list_store.set_value(row_iter, 12,
                                      entry.annotation["ifdb"]["starrating"])
            rating_pixbuf = self.get_rating_pixbuf(
                entry.annotation["ifdb"]["starrating"])
            self.list_store.set_value(row_iter, 11, rating_pixbuf)
            self.rating_store.add_item(
                entry.annotation["ifdb"]["starrating"], rating_pixbuf)
        if entry.bibliographic["description"] != biblio_info['Description']:
            if biblio_info['Description'] is not None:
                entry.bibliographic["description"] = biblio_info['Description']
            else:
                entry.bibliographic["description"] = ""
            self.list_store.set_value(row_iter, 13,
                                      entry.bibliographic["description"])
        if entry.format != biblio_info['Format']:
            entry.format = biblio_info['Format']
        # TODO: support IFID lists
        if entry.ifid_list[0] != biblio_info['IFID']:
            entry.ifid_list[0] = biblio_info['IFID']
        if entry.annotation.get("ifdb") is None and biblio_info['TUID']:
            entry.add_annotation("ifdb", {"tuid": biblio_info['TUID']})
        elif entry.annotation["ifdb"].get("tuid") != biblio_info['TUID']:
            entry.annotation["ifdb"]["tuid"] = biblio_info['TUID']
        self.list_store.set_value(row_iter, 15, entry)

    def get_rating_pixbuf(self, rating):
        '''This method fetches a star rating image given a text description
        of the rating (ie "1", "2.5", etc).

        '''
        rating_str = rating.replace('.', '-')
        rating_file = os.path.join(os.path.dirname(__file__), '..', '..', '..',
                                   'data', 'stars_{0}.png'.format(rating_str))
        rating_image = Gtk.Image.new_from_file(rating_file)
        return rating_image.get_pixbuf()

    def parse_list_str(self, list_str):
        '''This method parses strings which may be lists separated by commas
        or slashes, such as author lists or genre lists.

        '''
        itemized = list_str.split(',')
        final_list = itemized
        for item in itemized:
            for linker in [' and ', '&', '/']:
                if linker in item:
                    item_split = item.split(linker)
                    final_list.remove(item)
                    final_list.extend(item_split)
        return final_list

    def mark_story_played(self, row_iter, played="True"):
        '''This method handles marking a story as having been played.

        '''
        self.list_store.set_value(row_iter, 0, played)
        story = self.list_store.get_value(row_iter, 15)
        story.annotation["grotesque"]["played"] = played
        self.list_store.set_value(row_iter, 15, story)
        if played == "True":
            weight = Pango.Weight.NORMAL
        else:
            weight = Pango.Weight.BOLD
        self.list_store.set_value(row_iter, 16, weight)

    def get_story(self, row_iter):
        '''This is a small helper method to return a story file.

        '''
        return self.list_store.get_value(row_iter, 15)

    def story_in_library(self, story_file):
        '''This is a small helper method to check if a story file is already in
        the library.

        '''
        return story_file in self._files

    def load_ifiction(self, file_handle):
        '''This method loads a library from an ifiction XML file.

        '''
        xml = file_handle.read()
        stories = ifiction.get_stories_from_ifiction(xml)
        for story in stories:
            self.add_entry(story)

    def save_to_ifiction(self, file_handle, grotesque_version):
        '''This method saves the library to an ifiction XML file.

        '''
        doc = ifiction.create_ifiction_dom()
        list_iter = self.list_store.get_iter_first()
        while list_iter is not None:
            story = self.list_store.get_value(list_iter, 15)
            story.colophon["generator"] = u"Grotesque"
            story.colophon["generatorversion"] = unicode(grotesque_version)
            story.colophon["originated"] = unicode(time.strftime("%x"))
            story_node = story.to_ifiction_story_node(doc)
            ifiction.merge_story(doc, story_node)
            list_iter = self.list_store.iter_next(list_iter)
        xml = doc.toprettyxml(indent="\t", encoding="UTF-8")
        file_handle.write(xml)

    def import_from_ifiction(self, file_handle, fetch_cover, parent):
        try:
            doc = ifiction.get_ifiction_dom(file_handle.read())
        except:
            raise IOError("File is not an IFiction file")
        story_nodes = ifiction.get_all_stories(doc)
        stories = []
        for node in story_nodes:
            story = ifstory.IFStory(ific_story_node=node)
            stories.append(story)
            if (story.annotation.get("grotesque") and
                    story.annotation["grotesque"].get("localfile")):
                story.story_file = story.annotation["grotesque"]["localfile"]
        if fetch_cover:
            import_dialog = ProgressDialog("Importing covers from IFDB...",
                                           parent)
            import_thread = CoverImportThread(stories, import_dialog)
            importing = import_thread.import_covers()
            GObject.idle_add(importing.next)
            import_response = import_dialog.run()
            import_dialog.destroy()
            if (import_response == Gtk.ResponseType.REJECT or
                    import_response == Gtk.ResponseType.DELETE_EVENT):
                import_thread.stop()
        for story in stories:
            self.add_entry(story)

    def serialize_liststore(self):
        '''This method creates a serializable version of the list store.

        '''
        rows = []
        row_iter = self.list_store.get_iter_first()
        while row_iter:
            row = self.list_store.get(row_iter, 0, 1, 2, 3, 4, 5, 6,
                                      7, 8, 9, 10, 11, 12, 13, 14, 15,
                                      16)
            row = list(row)
            row[11] = None
            story = row[15]
            row[15] = [story.to_ifiction(indent=""), story.cover.data,
                       story.cover.img_format, story.cover.height,
                       story.cover.width, story.story_file]
            if row[16] == Pango.Weight.BOLD:
                row[16] = 'bold'
            else:
                row[16] = 'normal'
            rows.append(row)
            row_iter = self.list_store.iter_next(row_iter)
        return rows

    def deserialize_liststore(self, data, update):
        '''This method takes the serialized data and puts it back into the list
        store.

        '''
        for row in data:
            rating = row[12]
            pixbuf = self.get_rating_pixbuf(rating)
            row[11] = pixbuf
            story = row[15]
            if len(story) == 6:
                tmpstory = ifstory.IFStory()
                story_node = ifiction.get_ifiction_dom(story[0])
                tmpstory.load_from_ifiction(story_node)
                tmpstory.cover.data = story[1]
                tmpstory.cover.img_format = story[2]
                tmpstory.cover.height = story[3]
                tmpstory.cover.width = story[4]
                tmpstory.story_file = story[5]
                row[15] = tmpstory
            else:
                # TODO: can remove once migrated to db
                tmpstory = ifstory.IFStory()
                tmpstory.story_file = story[0]
                tmpstory.ifid_list = [story[1]]
                tmpstory.add_annotation("ifdb", {"tuid": story[2]})
                tmpstory.format = story[3]
                if story[4]:
                    tmpstory.bibliographic["title"] = story[4].decode('UTF-8')
                if story[5]:
                    tmpstory.bibliographic["author"] = story[5].decode('UTF-8')
                if story[6]:
                    tmpstory.bibliographic["language"] = story[6].decode('UTF-8')
                if story[7]:
                    tmpstory.bibliographic["headline"] = story[7].decode('UTF-8')
                if story[8]:
                    tmpstory.bibliographic["firstpublished"] = story[8].decode('UTF-8')
                if story[9]:
                    tmpstory.bibliographic["genre"] = story[9].decode('UTF-8')
                if story[10]:
                    tmpstory.bibliographic["group"] = story[10].decode('UTF-8')
                if story[11]:
                    tmpstory.bibliographic["description"] = story[11].decode('UTF-8')
                if story[12]:
                    tmpstory.bibliographic["series"] = story[12].decode('UTF-8')
                if story[13]:
                    tmpstory.bibliographic["seriesnumber"] = story[13].decode('UTF-8')
                if story[14]:
                    tmpstory.bibliographic["forgiveness"] = story[14].decode('UTF-8')
                tmpstory.add_annotation("ifdb",
                                        {"starrating": story[15]})
                tmpstory.add_annotation("grotesque",
                                        {"played": str(story[16])})
                if story[17]:
                    with open(story[17]) as cover_file:
                        cover_data = cover_file.read()
                        if cover_data is None:
                            return
                        width, height = get_jpeg_dim(cover_data)
                        if width is not None and height is not None:
                            tmpstory.cover.data = cover_data
                            tmpstory.cover.img_format = "jpg"
                            tmpstory.cover.width = width
                            tmpstory.cover.height = height
                            tmpstory.cover.description = None
                        else:
                            width, height = get_png_dim(cover_data)
                            if width is not None and height is not None:
                                tmpstory.cover.data = cover_data
                                tmpstory.cover.img_format = "png"
                                tmpstory.cover.width = width
                                tmpstory.cover.height = height
                                tmpstory.cover.description = None
                row[15] = tmpstory
            if update == '0.9.3':
                story.insert(2, None)
                story.append(None)
            if row[16] == 'bold':
                row[16] = Pango.Weight.BOLD
            else:
                row[16] = Pango.Weight.NORMAL
            self.list_store.append(row)
            self._files.append(row[14])

    def save(self, file_handle, version):
        '''This method saves the library to a file via pickling serialized
        versions of its data.

        '''
        data = {'filters': {}, 'version': version}
        data['filters']['authors'] = self.author_store.serialize()
        data['filters']['years'] = self.year_store.serialize()
        data['filters']['genres'] = self.genre_store.serialize()
        data['filters']['groups'] = self.group_store.serialize()
        data['filters']['series'] = self.series_store.serialize()
        data['filters']['forgiveness'] = self.forgiveness_store.serialize()
        data['filters']['ratings'] = self.rating_store.serialize()
        data['library'] = self.serialize_liststore()
        cPickle.dump(data, file_handle, cPickle.HIGHEST_PROTOCOL)

    def load(self, file_handle):
        '''Deprecated

        '''
        stories = cPickle.load(file_handle)
        for story in stories:
            self.add_entry(story)
