# -*- coding: utf-8 -*-
#
#       libraryloadthread.py
#
#       Copyright © 2011, 2014 Brandon Invergo <brandon@invergo.net>
#
#       This file is part of Grotesque.
#
#       Grotesque is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Grotesque is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Grotesque.  If not, see <http://www.gnu.org/licenses/>.


import threading

from gi.repository import Gtk, Gdk


class LibraryLoadThread(threading.Thread):
    '''This class implements a thread which handles loading the library on
    program startup.

    The data is stored in a dict object keyed by 'library' and 'filters',
    containing rows in the library list store and the filter stores,
    respecively. The Library and FilterStore classes handle the serialization
    and deserialization of the data themselves.

    '''
    def __init__(self, library, data, dialog):
        threading.Thread.__init__(self)
        self.library = library
        self.data = data
        self.dialog = dialog
        self.stopthread = threading.Event()

    def stop(self):
        self.stopthread.set()

    def run(self):
        fraction = 0.0
        Gdk.threads_enter()
        self.library.deserialize_liststore(self.data['library'])
        fraction += 0.125
        self.dialog.progressbar.set_fraction(fraction)
        self.library.author_store.deserialize(self.data['filters']['authors'])
        fraction += 0.125
        self.dialog.progressbar.set_fraction(fraction)
        self.library.year_store.deserialize(self.data['filters']['years'])
        fraction += 0.125
        self.dialog.progressbar.set_fraction(fraction)
        self.library.genre_store.deserialize(self.data['filters']['genres'])
        fraction += 0.125
        self.dialog.progressbar.set_fraction(fraction)
        self.library.group_store.deserialize(self.data['filters']['groups'])
        fraction += 0.125
        self.dialog.progressbar.set_fraction(fraction)
        self.library.series_store.deserialize(self.data['filters']['series'])
        fraction += 0.125
        self.dialog.progressbar.set_fraction(fraction)
        self.library.forgiveness_store.deserialize(
            self.data['filters']['forgiveness'])
        fraction += 0.125
        self.dialog.progressbar.set_fraction(fraction)
        # The star rating pixbufs aren't actually serialized, so we have to
        # retrieve them via a Library class method. We store the pixbufs in
        # a dict keyed by the rating number and pass that to the FilterStore
        # for deserialization.
        pixbufs = dict([(rating, self.library.get_rating_pixbuf(rating)) for
                        rating in ['0', '0.5', '1', '1.5', '2', '2.5', '3',
                                   '3.5', '4', '4.5', '5']])
        self.library.rating_store.deserialize(self.data['filters']['ratings'],
                                              pixbufs)
        fraction += 0.125
        self.dialog.progressbar.set_fraction(fraction)
        self.dialog.response(Gtk.ResponseType.OK)
        Gdk.threads_leave()


def load_library(library, dialog, data, update):
    fraction = 0.0
    library.deserialize_liststore(data['library'], update)
    fraction += 0.125
    dialog.progressbar.set_fraction(fraction)
    yield True
    library.author_store.deserialize(data['filters']['authors'])
    fraction += 0.125
    dialog.progressbar.set_fraction(fraction)
    yield True
    library.year_store.deserialize(data['filters']['years'])
    fraction += 0.125
    dialog.progressbar.set_fraction(fraction)
    yield True
    library.genre_store.deserialize(data['filters']['genres'])
    fraction += 0.125
    dialog.progressbar.set_fraction(fraction)
    yield True
    library.group_store.deserialize(data['filters']['groups'])
    fraction += 0.125
    dialog.progressbar.set_fraction(fraction)
    yield True
    library.series_store.deserialize(data['filters']['series'])
    fraction += 0.125
    dialog.progressbar.set_fraction(fraction)
    yield True
    library.forgiveness_store.deserialize(
        data['filters']['forgiveness'])
    fraction += 0.125
    # dialog.progressbar.set_fraction(fraction)
    # The star rating pixbufs aren't actually serialized, so we have to
    # retrieve them via a Library class method. We store the pixbufs in
    # a dict keyed by the rating number and pass that to the FilterStore
    # for deserialization.
    pixbufs = dict([(rating, library.get_rating_pixbuf(rating)) for
                    rating in ['0', '0.5', '1', '1.5', '2', '2.5', '3',
                               '3.5', '4', '4.5', '5']])
    library.rating_store.deserialize(data['filters']['ratings'], pixbufs)
    fraction += 0.125
    dialog.progressbar.set_fraction(fraction)
    dialog.response(Gtk.ResponseType.OK)
    yield False
