# -*- coding: utf-8 -*-
#
#       storyimportthread.py
#
#       Copyright © 2011, 2012, 2014 Brandon Invergo <brandon@invergo.net>
#       Copyright © 2009, 2010 Per Liedman <per@liedman.net>
#
#       This file is part of Grotesque.
#
#       Grotesque is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Grotesque is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Grotesque.  If not, see <http://www.gnu.org/licenses/>.


import os
import time

from gi.repository import Gtk

from treatyofbabel import ifstory


class StoryImportThread():
    '''This class handles importing stories into the library in a threaded
    manner. This allows a progress bar to be displayed since the process can
    take some time.

    '''
    def __init__(self, filenames, settings, library, dialog,
                 grotesque_version):
        self.settings = settings
        self.filenames = filenames
        self.fetch_metadata = settings.get_fetch_metadata()
        self.fetch_coverart = settings.get_fetch_coverart()
        self.library = library
        self.dialog = dialog
        self.grotesque_version = grotesque_version
        self.stopthread = False
        self.fails = []

    def stop(self):
        self.stopthread = True

    def import_stories(self):
        self.msg = ''
        count = 0
        for filename in self.filenames:
            if self.stopthread:
                yield False
            # Ignore files that are already in the self.library.
            if self.library.story_in_library(filename):
                continue
            story = None
            basename = os.path.split(filename)[1]
            yield True
            self.dialog.info_label.set_text('Importing...')
            self.dialog.progressbar.set_text(basename)
            # Attempt to extract the metadata from the file.
            try:
                story = ifstory.IFStory(story_file=filename)
            except:
                self.msg = ''.join([self.msg,
                                    'Failed to import "{0}".\n\n'.format(
                                        os.path.split(filename)[1])])
                continue
            yield True
            # Add Grotesque-specific annotation
            annot = {'played': 'False',
                     'imported': time.strftime('%x'),
                     'localfile': filename}
            story.add_annotation('grotesque', annot)
            story.colophon['generator'] = 'Grotesque'
            story.colophon['generatorversion'] = self.grotesque_version
            story.colophon['originated'] = time.strftime('%x')
            # We have the IFID, but we don't have a title, so we should
            # fetch metadata from IFDB.
            if (story.bibliographic["author"] == "" and
                    story.bibliographic["title"] == ""):
                # Only fetch if the user has specified to do so in the
                # self.settings.
                if self.fetch_metadata:
                    progress_text = \
                        "{0}: Getting metadata from IFDB".format(basename)
                    self.dialog.progressbar.set_text(progress_text)
                    # Fill the story object with metadata from IFDB. Keep
                    # the XML file so we don't have to request it from the
                    # server again if we're going to download the cover
                    # art.
                    try:
                        story.load_from_ifdb(fetch_cover=self.fetch_coverart)
                    except:
                        self.fails.append(story)
                    yield True
                # If, after IFDB, we still don't have metadata, add
                # the file to the list of failures.
                if (story.bibliographic["author"] == "" and
                        story.bibliographic["title"] == ""):
                    story.bibliographic["title"] = os.path.splitext(
                        basename)[0]
                    story.bibliographic["author"] = "Unknown"
                    if story not in self.fails:
                        self.fails.append(story)
                    count = count + 1
                    self.dialog.progressbar.set_fraction(
                        float(count) / float(len(self.filenames)))
                    yield True
                    continue
            # Otherwise, add the story to the self.library.
            self.library.add_entry(story)
            count = count + 1
            # Update the progress bar.
            self.dialog.progressbar.set_fraction(
                float(count) / float(len(self.filenames)))
            yield True
        self.dialog.response(Gtk.ResponseType.OK)
        yield False


class CoverImportThread():
    '''This class handles importing stories into the library in a threaded
    manner. This allows a progress bar to be displayed since the process can
    take some time.

    '''
    def __init__(self, stories, dialog):
        self.stories = stories
        self.dialog = dialog
        self.stopthread = False

    def stop(self):
        self.stopthread = True

    def import_covers(self):
        self.msg = ''
        count = 0
        self.dialog.info_label.set_text("Fetching covers from IFDB...")
        for story in self.stories:
            if self.stopthread:
                yield False
            title = story.bibliographic["title"]
            self.dialog.progressbar.set_text(title)
            story.load_cover_from_ifdb()
            count += 1
            self.dialog.progressbar.set_fraction(
                float(count) / float(len(self.stories)))
            yield True
        self.dialog.response(Gtk.ResponseType.OK)
        yield False
