# -*- coding: utf-8 -*-
#
#       storyremovethread.py
#
#       Copyright © 2011, 2014 Brandon Invergo <brandon@invergo.net>
#
#       This file is part of Grotesque.
#
#       Grotesque is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Grotesque is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Grotesque.  If not, see <http://www.gnu.org/licenses/>.


import threading

from gi.repository import Gtk, Gdk


class StoryRemoveThread(threading.Thread):
    '''This class handles removing stories from the library in a threaded
    manner.  This is predominantly in order to provide a progress bar,
    since this process can take a couple seconds if many stories are
    selected.

    '''
    def __init__(self, library, row_iters, dialog):
        threading.Thread.__init__(self)
        self.library = library
        self.row_iters = row_iters
        self.dialog = dialog
        self.stopthread = threading.Event()

    def stop(self):
        self.stopthread.set()

    def run(self):
        for n, row_iter in enumerate(self.row_iters):
            Gdk.threads_enter()
            # If the current iter somehow points to a row that doesn't exist,
            # continue.
            if not row_iter:
                continue
            # Get the story filename (column 14) and all of the bibliographic
            # information that is used for filtering.
            cols = self.library.list_store.get(row_iter, 1, 2, 5, 6, 7, 8, 10,
                                               14)
            # Remove the entry from the library list store
            self.library.list_store.remove(row_iter)
            # Remove the filename from the list of files in the library
            self.library._files.remove(cols[7])
            # Remove the bibliographic information from the filters.  The
            # FilterStore class handles keeping count of the number of
            # instances of each item in the library and whether or not the
            # item should actually be cleared from the filter.
            self.library.author_store.remove_item(cols[1])
            year = cols[2].split("-")[0]
            self.library.year_store.remove_item(year)
            self.library.genre_store.remove_item(cols[3])
            self.library.group_store.remove_item(cols[4])
            self.library.series_store.remove_item(cols[5])
            self.library.forgiveness_store.remove_item(cols[6])
            # Advance the progress bar.
            self.dialog.progressbar.set_fraction(float(n + 1) /
                                                 float(len(self.row_iters)))
            Gdk.threads_leave()
        Gdk.threads_enter()
        self.dialog.response(Gtk.ResponseType.OK)
        Gdk.threads_leave()


def remove_stories(library, row_iters, dialog):
    for n, row_iter in enumerate(row_iters):
        # If the current iter somehow points to a row that doesn't exist,
        # continue.
        if not row_iter:
            continue
        # Get the story filename (column 14) and all of the bibliographic
        # information that is used for filtering.
        cols = library.list_store.get(row_iter, 1, 2, 5, 6, 7, 8, 10, 12,
                                      14)
        # Remove the entry from the library list store
        library.list_store.remove(row_iter)
        # Remove the filename from the list of files in the library
        library._files.remove(cols[8])
        # Remove the bibliographic information from the filters.  The
        # FilterStore class handles keeping count of the number of
        # instances of each item in the library and whether or not the
        # item should actually be cleared from the filter.
        authors = library.parse_list_str(cols[1])
        for author in authors:
            author_real_name = author.split('(')[0]
            if author_real_name == '':
                continue
            library.author_store.remove_item(author_real_name.strip())
        year = cols[2].split("-")[0]
        library.year_store.remove_item(year)
        genres = library.parse_list_str(cols[3])
        for genre in genres:
            library.genre_store.remove_item(genre.strip())
        library.group_store.remove_item(cols[4])
        library.series_store.remove_item(cols[5])
        library.forgiveness_store.remove_item(cols[6])
        library.rating_store.remove_item(cols[7])
        # Advance the progress bar.
        dialog.progressbar.set_fraction(float(n + 1) /
                                        float(len(row_iters)))
        yield True
    dialog.response(Gtk.ResponseType.OK)
    yield False
