# -*- coding: utf-8 -*-
#
#       settings.py
#
#       Copyright © 2011, 2014, 2015 Brandon Invergo <brandon@invergo.net>
#       Copyright © 2009, 2010 Per Liedman <per@liedman.net>
#
#       This file is part of Grotesque.
#
#       Grotesque is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Grotesque is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Grotesque.  If not, see <http://www.gnu.org/licenses/>.

import os
import platform
from ConfigParser import SafeConfigParser, NoOptionError


class Settings:
    '''This class handles reading from and writing to the program's
    configuration file.

    '''
    DEFAULT_DIMENSIONS = (1024, 768)

    def __init__(self):
        self.config = SafeConfigParser()
        self.config.add_section('General')
        self.config.add_section('Launchers')
        self.config.add_section('File_exts')
        self.config.add_section('Directories')
        self.config.add_section('Window')
        # Get the user's home directory.
        if platform.system() == 'Windows':
            homedir = os.path.expanduser('~')
            appdir = '.grotesque'
            self.configdir = os.path.join(homedir, appdir)
            self.settings_filename = os.path.join(self.configdir,
                                                  'grotesque.cfg')
            self.library_filename = os.path.join(self.configdir,
                                                 'local_library.dat')
            self.coverart_dir = os.path.join(self.configdir, 'covers')
        else:
            home_configdir = os.getenv('XDG_CONFIG_HOME')
            if home_configdir is None:
                self.configdir = os.path.join(os.path.expanduser('~'),
                                              '.config', 'grotesque')
            else:
                self.configdir = os.path.join(home_configdir, 'grotesque')
            home_datadir = os.getenv('XDG_DATA_HOME')
            if home_datadir is None:
                datadir = os.path.join(os.path.expanduser('~'), '.local',
                                       'share', 'grotesque')
            else:
                datadir = os.path.join(home_datadir, 'grotesque')
        # Create a directory for Grotesque.
        # Define program's files.
            self.settings_filename = os.path.join(self.configdir,
                                                  'grotesque.cfg')
            self.library_filename = os.path.join(datadir,
                                                 'local_library.dat')
            self.coverart_dir = os.path.join(datadir, 'covers')

    def load(self):
        return len(self.config.read(self.settings_filename)) > 0

    def save(self):
        with open(self.settings_filename, 'w') as f:
            self.config.write(f)

    def set_fetch_metadata(self, fetch_metadata):
        self.config.set('General', 'FetchMetadata', str(fetch_metadata))

    def get_fetch_metadata(self):
        str_to_bool = {'True': True, 'False': False}
        try:
            fetch = self.config.get('General', 'FetchMetadata')
            return str_to_bool[fetch]
        except NoOptionError:
            return True

    def set_fetch_coverart(self, fetch_coverart):
        self.config.set('General', 'FetchCoverArt', str(fetch_coverart))

    def get_fetch_coverart(self):
        str_to_bool = {'True': True, 'False': False}
        try:
            fetch = self.config.get('General', 'FetchCoverArt')
            return str_to_bool[fetch]
        except NoOptionError:
            return True

    def set_disp_coverart(self, disp_coverart):
        self.config.set('General', 'DispCoverArt', str(disp_coverart))

    def get_disp_coverart(self):
        str_to_bool = {'True': True, 'False': False}
        try:
            disp = self.config.get('General', 'DispCoverArt')
            return str_to_bool[disp]
        except NoOptionError:
            return True

    def get_launcher(self, if_format):
        try:
            return self.config.get('Launchers', if_format)
        except NoOptionError:
            raise Exception('No interpreter configured for format {0}.'.format(
                if_format))

    def get_launchers(self):
        return self.config.items('Launchers')

    def set_launcher(self, format, path):
        self.config.set('Launchers', format, path)

    def remove_launcher(self, format):
        self.config.remove_option('Launchers', format)
        self.config.remove_option('File_exts', format)

    def get_file_exts(self, if_format):
        try:
            return self.config.get('File_exts', if_format)
        except NoOptionError:
            if if_format == 'adrift':
                return 'taf,blorb'
            elif if_format == 'agt':
                return 'agx'
            elif if_format == 'alan2':
                return 'acd,dat'
            elif if_format == 'alan3':
                return 'a3c,a3r'
            elif if_format == 'executable':
                return 'exe'
            elif if_format == 'glulx':
                return 'ulx,blb,gblorb,blorb'
            elif if_format == 'hugo':
                return 'hex'
            elif if_format == 'level9':
                return 'l9,sna'
            elif if_format == 'magscrolls':
                return 'mag'
            elif if_format == 'tads2':
                return 'gam'
            elif if_format == 'tads3':
                return 't3'
            elif if_format == 'zcode':
                exts = ['z{0}'.format(version) for version in range(3, 9)]
                exts.append('zblorb')
                return ','.join(exts)
            else:
                return ''

    def set_file_exts(self, if_format, exts):
        self.config.set('File_exts', if_format, exts)

    def get_all_exts(self):
        exts = self.config.items('File_exts')
        if len(exts) == 0:
            for if_format in ['adrift', 'agt', 'alan2', 'alan3', 'executable',
                              'glulx', 'hugo', 'level9', 'magscrolls', 'tads2',
                              'tads3', 'zcode']:
                self.set_file_exts(if_format, self.get_file_exts(if_format))
            self.save()
        return self.config.items('File_exts')

    def get_format_from_ext(self, ext):
        if_formats = []
        all_exts = self.config.items('File_exts')
        for if_format, format_ext in all_exts:
            if ext in format_ext:
                if_formats.append(if_format)
        if len(if_formats) != 1:
            return None
        else:
            return if_formats[0]

    def get_game_dir(self):
        try:
            return self.config.get('Directories', 'Games')
        except NoOptionError:
            return os.getcwd()

    def set_game_dir(self, dir):
        self.config.set('Directories', 'Games', dir)

    def get_library_filename(self):
        return self.library_filename

    def set_window_size(self, size):
        width, height = size
        self.config.set('Window', 'Width', str(width))
        self.config.set('Window', 'Height', str(height))

    def get_window_size(self):
        try:
            width = int(self.config.get('Window', 'Width'))
            height = int(self.config.get('Window', 'Height'))
            return (width, height)
        except NoOptionError:
            return self.DEFAULT_DIMENSIONS

    def set_vpaned_percentage(self, percentage):
        '''This method sets the position of the handle splitting the library
        area from the information area of the main window.

        '''
        self.config.set('Window', 'VPanedPercentage', str(percentage))

    def get_vpaned_percentage(self):
        '''This method gets the position of the handle splitting the library
        area from the information area of the main window.

        '''
        try:
            return float(self.config.get('Window', 'VPanedPercentage'))
        except NoOptionError:
            return 0.625

    def set_filter_percentage(self, percentage):
        '''This method sets the position of the handle splitting the list of
        library filters from the list of library contents in the main window.

        '''
        self.config.set('Window', 'FilterPercentage', str(percentage))

    def get_filter_percentage(self):
        '''This method gets the position of the handle splitting the list of
        library filters from the list of library contents in the main window.

        '''
        try:
            return float(self.config.get('Window', 'FilterPercentage'))
        except NoOptionError:
            return 0.2

    def set_filter_type(self, filter_index):
        '''This method sets the type of library filter being used.

        '''
        self.config.set('Window', 'Filter', str(filter_index))

    def get_filter_type(self):
        '''This method gets the type of library filter to use.

        '''
        try:
            return int(self.config.get('Window', 'Filter'))
        except NoOptionError:
            return 0

    def set_column_width(self, title, width):
        '''This method sets the current width of a column of the library view.

        '''
        self.config.set('Window', '{0}_col_width'.format(title), str(width))

    def get_column_width(self, title):
        '''This method gets the width of a column of the library view.

        '''
        try:
            width = int(self.config.get(
                'Window', '{0}_col_width'.format(title)))
        except NoOptionError:
            return 150
        if width == 0:
            return 150
        else:
            return width

    def set_column_visible(self, title, visible):
        '''This method sets whether or not a column is visible.

        '''
        self.config.set('Window', '{0}_visible'.format(title), str(visible))

    def get_column_visible(self, title):
        '''This method gets whether or not a column should be visible.

        '''
        str_to_bool = {'True': True, 'False': False}
        try:
            vis = self.config.get('Window', '{0}_visible'.format(title))
            return str_to_bool[vis]
        except NoOptionError:
            return True
